/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: DBObserver.java,v 1.7 2003/08/07 20:13:21 vgritsenko Exp $
 */

package org.apache.xindice.core;

import org.apache.xindice.core.data.Key;
import org.apache.xindice.core.data.Record;
import org.apache.xindice.util.Configuration;

import org.w3c.dom.Document;

import java.util.Map;

/**
 * Observer for Xindice DB activities
 *
 * @version CVS $Revision: 1.7 $, $Date: 2003/08/07 20:13:21 $
 */
public abstract class DBObserver {

    private static final DBObserver NOOP = new DBObserver() {
        public void setDatabaseConfig(Database db, Map collections, Configuration cfg) {
        }

        public void setCollectionConfig(Collection col, Configuration cfg) {
        }

        public void flushDatabaseConfig(Database db, Configuration cfg) {
        }

        public void dropCollection(Collection col) throws DBException {
        }

        public void createCollection(Collection col) throws DBException {
        }

        public void putDocument(Collection col, Key key, Document document, boolean create) throws DBException {
        }

        public void loadDocument(Collection col, Record record, Document document) throws DBException {
        }

        public void dropDocument(Collection col, Key key) throws DBException {
        }
    };

    private static DBObserver instance = NOOP;

    /**
     * Sets the default observer instance
     */
    public static void setInstance(DBObserver obs) {
        instance = (null == obs) ? NOOP : obs;
    }

    /**
     * Returns the observer instance, must be non-null
     */
    public static DBObserver getInstance() {
        return instance;
    }

    /**
     * Called after Database.setConfig()
     */
    public abstract void setDatabaseConfig(Database db, Map collections, Configuration cfg);

    /**
     * Called after Collection.setConfig()
     */
    public abstract void setCollectionConfig(Collection col, Configuration cfg);

    /**
     * Called after Database.flushConfig()
     */
    public abstract void flushDatabaseConfig(Database db, Configuration cfg);

    /**
     * Called before Collection.drop()
     */
    public abstract void dropCollection(Collection col) throws DBException;

    /**
     * Called after Collection.create()
     */
    public abstract void createCollection(Collection col) throws DBException;

    /**
     * Called after Collection.putDocument()
     */
    public abstract void putDocument(Collection col, Key key, Document document, boolean create) throws DBException;

    /**
     * Called after Collection.getDocument()
     */
    public abstract void loadDocument(Collection col, Record record, Document document) throws DBException;

    /**
     * Called before Collection.remove(key)
     */
    public abstract void dropDocument(Collection col, Key key) throws DBException;

}
