/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 2003 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: UglyBrowser.java,v 1.11 2003/12/25 03:46:02 vgritsenko Exp $
 */

package org.apache.xindice.server;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.xindice.core.Collection;
import org.apache.xindice.core.DBException;
import org.apache.xindice.core.Database;
import org.apache.xindice.xml.TextWriter;

import org.w3c.dom.Document;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.OutputStream;
import java.util.StringTokenizer;

/**
 * Moved out of XindiceServlet for separation of display and server bootstrapping.
 *
 * @author kross
 * @version CVS $Revision: 1.11 $, $Date: 2003/12/25 03:46:02 $
 */
public class UglyBrowser {

    private static final Log log = LogFactory.getLog(UglyBrowser.class);

    /**
     *
     */
    public UglyBrowser() {
        super();
    }

    //    Get handling to provide simple HTTP access to the Xindice instance.
    public static void doGet(HttpServletRequest request, HttpServletResponse response, Database db) throws ServletException {
        try {
            String contextPath = request.getContextPath();
            StringBuffer sb = new StringBuffer();
            String path = request.getQueryString();
            sb.append("<html><body><center>");
            if (path == null) {
                sb.append("<h2>THIS IS AN UGLY DEBUG TOOL!</h2><p>");
                sb.append("To browse the database, follow the link ");
                sb.append("<a href=\"" + contextPath + "/?/" + db.getName() + "\">" + db.getName() + "</a>");
            } else {
                // we have something to chew on
                XPathPseudoParser parser = new XPathPseudoParser(path);

                /* building the page once we have all the information */
                sb.append("<table border=\"1\" width=\"90%\">\n");
                sb.append("<tr><td rowspan=\"1\" colspan=\"2\">" + getPathNavigation(parser, contextPath) + "</td></tr>\n");
                sb.append("<tr><td valign=\"top\">" + getHierarchy(parser, contextPath, db) + "</td>");
                sb.append("<td valign=\"top\">" + getDetailView(parser, db) + "</td></tr>\n");
                sb.append("</table>\n");
            }
            sb.append("</center>");
            sb.append("<p><font size=\"-1\">Ugly Browser version: $Revision: 1.11 $ </font>\n</p>");
            sb.append("</body></html>");

            response.setContentType("text/html; charset=utf-8");
            byte[] resultBytes = sb.toString().getBytes("utf-8");
            response.setContentLength(resultBytes.length);
            OutputStream output = response.getOutputStream();
            output.write(resultBytes);
            output.flush();
        } catch (Exception e) {
            throw new ServletException("argh!", e);
            //response.setStatus(HttpServletResponse.SC_NOT_FOUND);
            // TODO: need intelligent error handling.
            // throw new ServletException("Error while doGet method", e);
        }
    }

    protected static String getPathNavigation(XPathPseudoParser parser, String contextPath) {
        String path = parser.getQuery();
        StringBuffer result = new StringBuffer();
        StringTokenizer st = new StringTokenizer(path, "/");
        String currentPath = "<a href=\"" + contextPath + "/?";
        while (st.hasMoreTokens()) {
            String token = st.nextToken();
            if (token.indexOf("&") > 0) {
                token = token.substring(0, token.indexOf("&"));
            }
            currentPath = currentPath + "/" + token;
            result.append(currentPath + "\">" + token + "</a>&nbsp;-&nbsp;");

        }
        return result.toString();
    }

    protected static String getHierarchy(XPathPseudoParser parser, String contextPath, Database db) throws DBException, Exception {
        StringBuffer result = new StringBuffer();

        String path = parser.getPath();

        Collection col;
        if (!path.equals("")) {
            col = db.getCollection(parser.getPath());
        } else {
            col = db;
        }

        if (col == null) {
            result.append("Collection not found! " + parser.getPath());
            return result.toString();
        }

        String[] cols = col.listCollections();
        String dbLoc = parser.getDatabase();
        String parserPath = parser.getPath();
        StringBuffer baseHref = new StringBuffer();
        baseHref.append("<a href=\"" + contextPath + "/?" + dbLoc);
        if(parserPath.startsWith("/") || dbLoc.endsWith("/")) {
            baseHref.append(parserPath);
        } else {
            baseHref.append("/" + parserPath);
        }

        for (int i = 0; i < cols.length; i++) {
            result.append(baseHref);
            if(!baseHref.toString().endsWith("/")) {
                result.append("/");
            }
            result.append(cols[i]);
            result.append("\">");
            result.append(cols[i]);
            result.append("</a><br>");
        }

        try {
            String[] docs = col.listDocuments();
            for (int i = 0; i < docs.length; i++) {
                result.append(baseHref);
                result.append("&");
                result.append(docs[i]);
                result.append("\">");
                result.append(docs[i]);
                result.append("</a><br>");
            }
        } catch (DBException e) {
            // do nothing but this collection cannot store documents.
            if (log.isWarnEnabled()) {
                log.warn("ignored exception", e);
            }
        }

        return result.toString();
    }

    protected static String getDetailView(XPathPseudoParser parser, Database db) throws Exception {
        Collection col = db.getCollection(parser.getPath());

        String document = parser.getDocument();
        if (document == null) {
            return "";
        }

        Document doc = col.getDocument(document);
        if (doc == null) {
            return "Document '" + document + "' not found";
        }

        return "Document '" + document + "'<p>\n" 
            + escapeHtml(TextWriter.toString(doc));
    }

    /* simple function to escape the xml stuff that'll
     * not be displayed correctly in html.
     */
    static public String escapeHtml(String value) {
        StringBuffer buf = new StringBuffer();
        for (int i = 0; i < value.length(); i++) {
            char c = value.charAt(i);
            switch (c) {
                case '<' :
                    buf.append("&lt;");
                    break;
                case '>' :
                    buf.append("&gt;");
                    break;
                case '\'' :
                    buf.append("&apos;");
                    break;
                case '\"' :
                    buf.append("&quot;");
                    break;
                case '&' :
                    buf.append("&amp;");
                    break;
                default :
                    if (isLegalCharacter(c)) {
                        buf.append(c);
                    } else {
                        buf.append("&#");
                        buf.append(String.valueOf(c));
                        buf.append(";");
                    }
                    break;
            }
        }
        return buf.toString();
    }

    /**
     * Is the given character allowed inside an HTML document?
     */
    static public boolean isLegalCharacter(char c) {
        return (c > 0x07) && (c < 0x80);
    }

    public static class XPathPseudoParser {

        private String query;

        public XPathPseudoParser(String query) throws Exception {
            this.query = query;
        }

        public String getQuery() {
            return this.query;
        }

        public String getDatabase() {
            int max = query.indexOf("/", 1);
            if (max > 1) {
                return query.substring(0, max);
            } else {
                return query;
            }
        }

        public String getPath() {
            int min = query.indexOf("/", 1);
            if (min <= 0) {
                return "";
            }

            int max = query.indexOf("&");
            if (max > 1) {
                return query.substring(min + 1, max);
            } else {
                return query.substring(min + 1);
            }
            /*
            int pos = query.indexOf("&");
            if (pos > 0) {
               return query.substring(0, pos);
            } else {
               return query;
            }
            */
        }

        public String getDocument() {
            int pos = query.indexOf("&");
            if (pos > 0) {
                return query.substring(pos + 1);
            } else {
                return null;
            }
        }
    }
}
