/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: AddMultipleDocuments.java,v 1.9 2003/08/07 20:13:24 vgritsenko Exp $
 */

package org.apache.xindice.tools.command;

import org.apache.xindice.tools.XMLTools;

import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.util.Hashtable;

/**
 * AddMultipleDocuments.java is designed to let the user add several Documents
 * to an existing Collection at one time.
 *
 * Documents are added with an ID the same as the file name.
 * This command assumes that the collection being added to already exists!
 *
 * @version CVS $Revision: 1.9 $, $Date: 2003/08/07 20:13:24 $
 */
public class AddMultipleDocuments extends Command {

    public boolean execute(Hashtable table) throws Exception {

        Collection col = null;
        try {

            // Verify that user has supplied necessary arguments
            if (table.get(XMLTools.COLLECTION) == null) {
                System.out.println("ERROR : Collection and switch required");
                return false;
            }

            if (table.get(XMLTools.FILE_PATH) == "") {
                System.out.println("ERROR : Directory name and switch required");
                return false;
            }

            // Get a Collection reference to the collection
            String colstring = normalizeCollectionURI((String) table.get(XMLTools.COLLECTION),
                                                      (String) table.get(XMLTools.LOCAL));
            col = DatabaseManager.getCollection(colstring);
            if (col == null) {
                System.out.println("ERROR : Collection not found!");
                return false;
            }

            // Create a collection manager instance for the collection
            // CollectionManager colman = (CollectionManager)col.getService("CollectionManager",XMLDBAPIVERSION);

            // Get a File object for the Directory passed in
            File dir = new File((String) table.get(XMLTools.FILE_PATH));

            if (dir.isDirectory()) {

                String[] children = new String[]{};
                final String ext = (String) table.get(XMLTools.EXTENSION);

                // If the user supplied a file extension, filter on it, else use entire contents of the directory
                if (!ext.equals("")) {

                    children = dir.list(new FilenameFilter() {
                        public boolean accept(File none, String name) {
                            return name.endsWith("." + ext);
                        }
                    });
                } else if (ext.equals("")) {
                    children = dir.list();
                }

                System.out.println("Reading files from: " + dir.getName());

                // Loop over documents, adding to db
                for (int i = 0; i < children.length; i++) {

                    File files = new File(dir, children[i]);

                    if (files.isFile()) {
                        FileInputStream insr = new FileInputStream(files);
                        byte[] fileBuffer = new byte[(int) files.length()];
                        insr.read(fileBuffer);

                        try {

                            // Use the functionality already provided in Command.AddDocument to add this document
                            Command cmd = (Command) Class.forName("org.apache.xindice.tools.command.AddDocument").newInstance();
                            Hashtable localtable = new Hashtable();
                            String filepath = dir.getPath() + "/" + files.getName();

                            // Populate hashtable to pass to AddDocument class
                            localtable.put(XMLTools.COLLECTION, table.get(XMLTools.COLLECTION));
                            localtable.put(XMLTools.NAME_OF, files.getName());
                            localtable.put(XMLTools.FILE_PATH, filepath);

                            // Execute the class!
                            cmd.execute(localtable);

                        } catch (Exception e) {
                            System.out.println("Error Adding File: " + files.getName());
                            //System.out.println(e);
                            continue;
                        }
                    } else
                        continue;
                } // for loop
            }

        } finally {
            // Release all resources
            if (col != null) {
                col.close();
            }
        }

        return true;
    }

}

