/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: ExportTree.java,v 1.9 2003/12/22 14:02:44 vgritsenko Exp $
 */

package org.apache.xindice.tools.command;

import org.apache.xindice.tools.XMLTools;

import org.xmldb.api.DatabaseManager;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.ErrorCodes;
import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.modules.XMLResource;

import java.io.File;
import java.io.FileOutputStream;
import java.util.Hashtable;

/**
 * ExportTree.java is designed to take a Collection tree and create a Directory
 * tree from it.  Export tree requires a collection name and a file path to create the tree
 *
 * @version CVS $Revision: 1.9 $, $Date: 2003/12/22 14:02:44 $
 */
public class ExportTree extends Command {

    /**
     * TODO: Description of the Method
     *
     * @param table Description of Parameter
     * @return Description of the Returned Value
     */
    public boolean execute(Hashtable table) throws Exception {

        Collection col = null;

        if (table.get(XMLTools.FILE_PATH).equals("")) {
            System.out.println("ERROR: Directory name and switch required");
            return false;
        }

        if (table.get(XMLTools.COLLECTION) == null) {
            System.out.println("ERROR : Collection name and switch required");
            return false;
        }

        File fp = new File((String) table.get(XMLTools.FILE_PATH));

        System.out.println();

        if (table.get(XMLTools.COLLECTION) != null) {
            String parent = parentDir((String) table.get(XMLTools.COLLECTION));
            File dir = new File(fp, parent);

            System.out.println("Creating directory " + dir.getPath());

            dir.mkdir();

            process(dir, table, col);
        }

        return true;
    }

    /**
     * TODO: Description of the Method
     *
     * @param parent Description of Parameter
     * @return Description of the Returned Value
     */
    public String parentDir(String parent) {
        String dirName = "";
        String parName = parent;

        int idx = parName.lastIndexOf("/");

        if (idx != -1) {
            dirName = parName.substring(idx + 1);
            parName = parName.substring(0, idx);
        } else if (idx == -1) {
            dirName = parName;
        }
        return dirName;
    }

    /**
     * TODO: Description of the Method
     *
     * @param directory Description of Parameter
     * @param table Description of Parameter
     * @param col Description of Parameter
     * @return Description of the Returned Value
     */
    boolean process(File directory, Hashtable table, Collection col) throws Exception {
        try {
            String[] list = null;
            String collection = (String) table.get(XMLTools.COLLECTION);

            // Get a Collection reference to the collection
            String colstring = normalizeCollectionURI((String) table.get(XMLTools.COLLECTION),
                                                      (String) table.get(XMLTools.LOCAL));
            col = DatabaseManager.getCollection(colstring);
            if (col == null) {
                System.out.println("ERROR : Collection not found!");
                return false;
            }

            String[] files = null;
            try {
                files = col.listResources();
            } catch (XMLDBException e) {
                // Xindice will throw an error if there is no indexer
                if (e.errorCode != ErrorCodes.VENDOR_ERROR) {
                    throw e;
                }
            }

            if (files != null) {
                System.out.println("Extracting " + files.length + " files from " +
                                   (String) table.get(XMLTools.COLLECTION));
                for (int j = 0; j < files.length; j++) {
                    // TODO: BinaryResource support
                    XMLResource res =
                            (XMLResource) col.getResource(files[j]);
                    String content = (String) res.getContent();
                    FileOutputStream output =
                            new FileOutputStream(new File(directory, files[j]));

                    /* UTF8FIXED: as we omit encoding declaration in output XML
                     * for the moment, we MUST write file in UTF-8
                     */
                    output.write(content.getBytes("utf-8"));
                    output.close();
                }
            }

            list = col.listChildCollections();
            for (int i = 0; i < list.length; i++) {
                File file = new File(directory, list[i]);

                System.out.println("Creating directory " + file.getPath());
                file.mkdirs();

                table.put(XMLTools.COLLECTION, (collection + "/" + file.getName()));

                process(new File(directory + "//" + file.getName()), table, col);
            }

        } finally {
            if (col != null) {
                col.close();
            }
        }
        return true;
    }
}

