/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: SymbolDeserializer.java,v 1.7 2003/08/07 20:13:24 vgritsenko Exp $
 */

package org.apache.xindice.util;

import org.apache.xindice.xml.SymbolTable;
import org.apache.xindice.xml.SymbolTableSymbols;
import org.apache.xindice.xml.dom.DocumentImpl;

import org.w3c.dom.Document;

import java.util.Hashtable;

/**
 * SymbolDeserializer is a utility class for managing SymbolTables in
 * the client context of Wire Compression.
 *
 * @version CVS $Revision: 1.7 $, $Date: 2003/08/07 20:13:24 $
 */
public final class SymbolDeserializer {
    private static final SymbolTableSymbols hcSyms = SymbolTableSymbols.getInstance();

    private SymbolTable syms = null; // The Collection's SymbolTable
    private long lastMod = 0;        // Last time we caught a SymbolTable modification

    /**
     * convertToDocument converts the compressed Hashtable to
     * a DOM Document.
     *
     * @param buffer The Hashtable
     * @return The Document
     */
    public Document convertToDocument(Hashtable buffer) {
        SymbolTable s = getSymbols(buffer);
        return new DocumentImpl((byte[]) buffer.get("document"), s, null);
    }

    /**
     * getSymbols returns the current SymbolTable image for the
     * server-side Collection that is being managed.  The Symbol
     * table is shipped as part of an Hashtable when the server
     * has determined that its image of the SymbolTable is out of
     * date.  This method either returns the current image or
     * extracts the new image from the Hashtable.
     *
     * @param buffer The Hashtable
     * @return The Symbol Table
     */
    public SymbolTable getSymbols(Hashtable buffer) {
        //if ( ((Long) buffer.get("timestamp")).longValue() != lastMod ) {
        // lastMod = ((Long) buffer.get("timestamp")).longValue();

        Document doc = new DocumentImpl((byte[]) buffer.get("symbols"), hcSyms, null);

        if (syms == null) {
            syms = new SymbolTable();
        }

        synchronized (syms) {
            syms.streamFromXML(doc.getDocumentElement());
        }
        // }
        return syms;
    }

    /**
     * getLastModified returns the last modified time stamp of the
     * client's image of the managed Symbol Table.
     *
     * @return The last modified stamp
     */
    public long getLastModified() {
        return lastMod;
    }
}

