/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: CollectionTest.java,v 1.15 2003/12/26 02:41:58 vgritsenko Exp $
 */

package org.apache.xindice.integration.client.basic;

import org.apache.xindice.integration.client.AbstractXmlDbClientTest;
import org.apache.xindice.integration.client.XmlDbClientSetup;
import org.apache.xindice.core.FaultCodes;

import org.xmldb.api.base.XMLDBException;
import org.xmldb.api.base.Collection;
import org.xmldb.api.base.ErrorCodes;

import java.util.Arrays;
import java.util.List;
import java.util.Vector;

/**
 * @version CVS $Revision: 1.15 $, $Date: 2003/12/26 02:41:58 $
 * @author Vladimir R. Bossicard <vladimir@apache.org>
 */
public class CollectionTest extends AbstractXmlDbClientTest {

    public void testGetUnknownDatabase() throws Exception {
        try {
            Collection col = this.client.getCollection("doesnotexist");
            fail("Expected ErrorCodes.NO_SUCH_DATABASE (" + ErrorCodes.NO_SUCH_DATABASE + "), got collection: " + col);
        } catch (XMLDBException e) {
            if (e.errorCode != ErrorCodes.NO_SUCH_DATABASE) {
                fail("Expected ErrorCodes.NO_SUCH_DATABASE (" + ErrorCodes.NO_SUCH_DATABASE + "), got: " + e);
            }
        }
    }

    // This is just to make sure that the system is up and running.
    public void testListInitialCollections() throws Exception {
        String[] collections = this.client.listCollections(XmlDbClientSetup.INSTANCE_NAME);
        List col = new Vector(Arrays.asList(collections));
        if (collections.length == 2) {
            assertTrue(col.contains(XmlDbClientSetup.TEST_COLLECTION_NAME));
            assertTrue(col.contains("system"));
        } else if (collections.length == 3) {
            assertTrue(col.contains(XmlDbClientSetup.TEST_COLLECTION_NAME));
            assertTrue(col.contains("system"));
            assertTrue(col.contains("meta"));
        } else {
            fail("Initial number of collections should be 2, or 3 if meta enabled");
        }
    }

    public void testCreateNestedCollection() throws Exception {
        this.client.createCollection(TEST_COLLECTION_PATH, "nested");
        assertEquals(1, this.client.countCollections(TEST_COLLECTION_PATH));

        // Test create collection with path
        this.client.createCollection(TEST_COLLECTION_PATH, "nested/child2", "child2");

        this.client.createCollection(TEST_COLLECTION_PATH + "/nested", "child3");
        assertEquals(2, this.client.countCollections(TEST_COLLECTION_PATH + "/nested"));

        this.client.dropCollection(TEST_COLLECTION_PATH + "/nested", "child2");
        assertEquals(1, this.client.countCollections(TEST_COLLECTION_PATH + "/nested"));

        this.client.dropCollection(TEST_COLLECTION_PATH + "/nested", "child3");
        assertEquals(0, this.client.countCollections(TEST_COLLECTION_PATH + "/nested"));

        this.client.dropCollection(TEST_COLLECTION_PATH, "nested");
    }

    public void testCreateDuplicateCollection() throws Exception {
        this.client.createCollection(TEST_COLLECTION_PATH, "duplicate");
        try {
            this.client.createCollection(TEST_COLLECTION_PATH, "duplicate");
        } catch (XMLDBException e) {
            assertTrue(e.vendorErrorCode == FaultCodes.COL_DUPLICATE_COLLECTION);
        }

        this.client.dropCollection(TEST_COLLECTION_PATH, "duplicate");
    }

    public void testCreateCollectionEmptyName() throws Exception {
        try {
            this.client.createCollection(TEST_COLLECTION_PATH, "emptyname", "");
        } catch (XMLDBException e) {
            assertTrue("Error Code (Expected " + FaultCodes.COL_CANNOT_CREATE + ", got " + e.vendorErrorCode,
                       e.vendorErrorCode == FaultCodes.COL_CANNOT_CREATE);
            return;
        }
        fail();
    }

    public void testCreateCollectionNullName() throws Exception {
        try {
            this.client.createCollection(TEST_COLLECTION_PATH, "nullname", (String) null);
        } catch (XMLDBException e) {
            assertTrue("Error Code (Expected " + FaultCodes.COL_CANNOT_CREATE + ", got " + e.vendorErrorCode,
                       e.vendorErrorCode == FaultCodes.COL_CANNOT_CREATE);
            return;
        }
        fail();
    }

    public void testCreateCollectionInvalidName() throws Exception {
        try {
            this.client.createCollection(TEST_COLLECTION_PATH, "invalidname", "my/collection");
        } catch (XMLDBException e) {
            assertTrue("Error Code (Expected " + FaultCodes.COL_CANNOT_CREATE + ", got " + e.vendorErrorCode,
                       e.vendorErrorCode == FaultCodes.COL_CANNOT_CREATE);
            return;
        }
        fail();
    }

    public void testDropCollectionTwice() throws Exception {
        this.client.createCollection(TEST_COLLECTION_PATH, "droptwice");

        this.client.dropCollection(TEST_COLLECTION_PATH, "droptwice");
        try {
            this.client.dropCollection(TEST_COLLECTION_PATH, "droptwice");
        } catch (XMLDBException e) {
            assertTrue("Error Code (Expected " + FaultCodes.COL_COLLECTION_NOT_FOUND + ", got " + e.vendorErrorCode,
                       e.vendorErrorCode == FaultCodes.COL_COLLECTION_NOT_FOUND);
            return;
        }
        fail();
    }

    public void testDropCollectionNullName() throws Exception {
        try {
            this.client.dropCollection(TEST_COLLECTION_PATH, null);
        } catch (XMLDBException e) {
            assertTrue("Error Code (Expected " + FaultCodes.COL_COLLECTION_NOT_FOUND + ", got " + e.vendorErrorCode,
                       e.vendorErrorCode == FaultCodes.COL_COLLECTION_NOT_FOUND);
            return;
        }
        fail();
    }

    public void testDropCollectionEmptyName() throws Exception {
        try {
            this.client.dropCollection(TEST_COLLECTION_PATH, "");
        } catch (XMLDBException e) {
            assertTrue(e.vendorErrorCode == FaultCodes.COL_COLLECTION_NOT_FOUND);
            return;
        }
        fail("Dropping a collection with empty name should throw an exception.");
    }

    public void testGetCollectionCount() throws Exception {
        this.client.createCollection(TEST_COLLECTION_PATH, "count");
        assertEquals(1, this.client.countCollections(TEST_COLLECTION_PATH));

        this.client.dropCollection(TEST_COLLECTION_PATH, "count");
        assertEquals(0, this.client.countCollections(TEST_COLLECTION_PATH));
    }

    public void testListCollections() throws Exception {
        String[] collections = this.client.listCollections(TEST_COLLECTION_PATH);
        assertEquals(0, collections.length);

        this.client.createCollection(TEST_COLLECTION_PATH, "child1");
        this.client.createCollection(TEST_COLLECTION_PATH, "child2");

        collections = this.client.listCollections(TEST_COLLECTION_PATH);
        assertEquals(2, collections.length);
        List col = new Vector(Arrays.asList(collections));
        assertTrue(col.contains("child1"));
        assertTrue(col.contains("child2"));

        this.client.dropCollection(TEST_COLLECTION_PATH, "child1");
        collections = this.client.listCollections(TEST_COLLECTION_PATH);
        assertEquals(1, collections.length);
        assertEquals("child2", collections[0]);

        this.client.dropCollection(TEST_COLLECTION_PATH, "child2");

        collections = this.client.listCollections(TEST_COLLECTION_PATH);
        assertEquals(0, collections.length);
    }

    public void testGetName() throws Exception {
        this.client.createCollection(TEST_COLLECTION_PATH, "getname");
        assertEquals("getname", this.client.getName(TEST_COLLECTION_PATH + "/getname"));
        this.client.dropCollection(TEST_COLLECTION_PATH, "getname");
    }

    public void testGetNameEndWithSlash() throws Exception {
        this.client.createCollection(TEST_COLLECTION_PATH, "getname");
        assertEquals("getname", this.client.getName(TEST_COLLECTION_PATH + "/getname/"));
        this.client.dropCollection(TEST_COLLECTION_PATH, "getname");
    }

    public void testGetParentCollection() throws Exception {
        Collection col = this.client.createCollection(TEST_COLLECTION_PATH, "childcol");

        Collection parent = col.getParentCollection();
        assertNotNull(parent);
        assertEquals("current", parent.getName());

        parent = parent.getParentCollection();
        assertNotNull(parent);
        assertEquals("testing", parent.getName());

        parent = parent.getParentCollection();

        // Must return root database collection
        assertNotNull(parent);
        assertEquals("", parent.getName());

        this.client.dropCollection(TEST_COLLECTION_PATH, "childcol");
    }

    public void testGetCollectionUnknown() throws Exception {
        assertNull(this.client.getCollection(TEST_COLLECTION_PATH + "/unknown"));
    }

    public void testChildCollectionCount() throws Exception {
        Collection col = this.client.createCollection(TEST_COLLECTION_PATH, "childcol");
        assertEquals(0, col.getChildCollectionCount());

        this.client.createCollection(TEST_COLLECTION_PATH, "colcount");
        assertEquals(0, col.getChildCollectionCount());

        this.client.dropCollection(TEST_COLLECTION_PATH, "colcount");
        assertEquals(0, col.getChildCollectionCount());
    }
}
