/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: DocumentTest.java,v 1.8 2003/12/24 15:23:53 vgritsenko Exp $
 */

package org.apache.xindice.integration.client.basic;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringWriter;
import java.io.Writer;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.parsers.SAXParserFactory;

import org.apache.xindice.integration.client.AbstractXmlDbClientTest;
import org.apache.xindice.tools.command.StringSerializer;
import org.custommonkey.xmlunit.XMLAssert;
import org.xml.sax.ContentHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xmldb.api.base.XMLDBException;

/**
 * @version CVS $Revision: 1.8 $, $Date: 2003/12/24 15:23:53 $
 * @author Vladimir R. Bossicard <vladimir@apache.org>
 */
public class DocumentTest extends AbstractXmlDbClientTest {

   public void testInsertDocument() throws Exception {
      this.client.insertDocument(TEST_COLLECTION_PATH, "testdoc", "<test>test data</test>");
      assertEquals(1, this.client.countDocument(TEST_COLLECTION_PATH));

      this.client.removeDocument(TEST_COLLECTION_PATH, "testdoc");
      assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
   }

   public void testRemoveDocumentInvalidName() throws Exception {
      try {
         this.client.removeDocument(TEST_COLLECTION_PATH, "invalidname");
      } catch (XMLDBException e) {
         //assertTrue( e.getMessage().lastIndexOf( "Duplicate Collection" ) > 0 );
         return;
      }
   }

   public void testRemoveDocumentEmptyName() throws Exception {
      try {
         this.client.removeDocument(TEST_COLLECTION_PATH, "");
      } catch (XMLDBException e) {
         //assertTrue( e.getMessage().lastIndexOf( "Duplicate Collection" ) > 0 );
         return;
      }
   }

   public void testRemoveDocumentNullName() throws Exception {
      try {
         this.client.removeDocument(TEST_COLLECTION_PATH, null);
      } catch (XMLDBException e) {
         //assertTrue( e.getMessage().lastIndexOf( "Duplicate Collection" ) > 0 );
         return;
      }
   }

   public void testGetDocument() throws Exception {
      String testDocument = "<?xml version=\"1.0\"?>\n<data><test>test data</test></data>";
      this.client.insertDocument(TEST_COLLECTION_PATH, "doc1", testDocument);

      String doc = this.client.getDocument(TEST_COLLECTION_PATH, "doc1");
      assertNotNull(doc);
		assertXMLEqual(testDocument, doc);

      this.client.removeDocument(TEST_COLLECTION_PATH, "doc1");
   }

	public void testGetDocumentAsSAX() throws Exception {
		String testDocument = "<?xml version=\"1.0\"?>\n<data><test>test data</test></data>";
		this.client.insertDocument(TEST_COLLECTION_PATH, "doc1", testDocument);

		final StringWriter out = new StringWriter();
		final ContentHandler serializer = createSerializer(out);

		this.client.getDocumentAsSax(TEST_COLLECTION_PATH, "doc1", serializer);
		String doc = out.toString();
		
		assertNotNull(doc);
		assertXMLEqual(testDocument, doc);

		this.client.removeDocument(TEST_COLLECTION_PATH, "doc1");
	}	
	
   public void testGetInexistantDocument() throws Exception {
      String doc = this.client.getDocument(TEST_COLLECTION_PATH, "ghostdoc");
      assertNull(doc);
   }

   public void testGetDocumentEmptyName() throws Exception {
      String doc = this.client.getDocument(TEST_COLLECTION_PATH, "");
      assertNull(doc);
   }

   public void testGetDocumentNullName() throws Exception {
      String doc = this.client.getDocument(TEST_COLLECTION_PATH, null);
      assertNull(doc);
   }

   public void testListDocuments() throws Exception {
      String[] documents = this.client.listDocuments(TEST_COLLECTION_PATH);
      assertEquals(0, this.client.listDocuments(TEST_COLLECTION_PATH).length);

      this.client.insertDocument(TEST_COLLECTION_PATH, "doc1", "<data>\n<test>\ntest data</test>\n</data>");
      documents = this.client.listDocuments(TEST_COLLECTION_PATH);
      assertEquals(1, documents.length);
      assertEquals("doc1", documents[0]);

      this.client.insertDocument(TEST_COLLECTION_PATH, "doc2", "<data>\n<test>\ntest data2</test>\n</data>");
      documents = this.client.listDocuments(TEST_COLLECTION_PATH);
      assertEquals(2, documents.length);
      assertEquals("doc1", documents[0]);
      assertEquals("doc2", documents[1]);

      this.client.removeDocument(TEST_COLLECTION_PATH, "doc1");
      documents = this.client.listDocuments(TEST_COLLECTION_PATH);
      assertEquals(1, documents.length);
      assertEquals("doc2", documents[0]);

      this.client.removeDocument(TEST_COLLECTION_PATH, "doc2");
      documents = this.client.listDocuments(TEST_COLLECTION_PATH);
      assertEquals(0, documents.length);
   }

   public void testGetDocumentCount() throws Exception {
      assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));

      this.client.insertDocument(TEST_COLLECTION_PATH, "doc1", "<data>\n<test>\ntest data</test>\n</data>");
      assertEquals(1, this.client.countDocument(TEST_COLLECTION_PATH));

      this.client.insertDocument(TEST_COLLECTION_PATH, "doc2", "<data>\n<test>\ntest data2</test>\n</data>");
      assertEquals(2, this.client.countDocument(TEST_COLLECTION_PATH));

      this.client.removeDocument(TEST_COLLECTION_PATH, "doc1");
      assertEquals(1, this.client.countDocument(TEST_COLLECTION_PATH));

      this.client.removeDocument(TEST_COLLECTION_PATH, "doc2");
      assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
   }
   
   public void testDocumentWithNameSpaces() throws Exception
   {
		assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
		
		final String testDocument = fetchTestDocument("namespaces.xml");
		this.client.insertDocument(TEST_COLLECTION_PATH, "namespaces.xml", testDocument);

		String doc = this.client.getDocument(TEST_COLLECTION_PATH, "namespaces.xml");
		assertNotNull(doc);
		assertXMLEqual(testDocument, doc);

		this.client.removeDocument(TEST_COLLECTION_PATH, "namespaces.xml");
		assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
   }
   
   public void testDocumentWithNameSpacesSAX() throws Exception
   {
		assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
		
		final String testDocument = fetchTestDocument("namespaces.xml");
		this.client.insertDocument(TEST_COLLECTION_PATH, "namespaces.xml", testDocument);

		final StringWriter out = new StringWriter();
		final ContentHandler serializer = createSerializer(out);

		this.client.getDocumentAsSax(TEST_COLLECTION_PATH, "namespaces.xml", serializer);
		String doc = out.toString();

		assertNotNull(doc);
		assertXMLEqual(testDocument, doc);

		this.client.removeDocument(TEST_COLLECTION_PATH, "namespaces.xml");
		assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
   }

	public void testDocumentWithSimpleNameSpacePrefixes() throws Exception
	{
		assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));

		final String testDocument = fetchTestDocument("namespace-simpleprefixes.xml");
		this.client.insertDocument(TEST_COLLECTION_PATH, "namespaces.xml", testDocument);

		String doc = this.client.getDocument(TEST_COLLECTION_PATH, "namespaces.xml");
		assertNotNull(doc);
		assertXMLEqual(testDocument, doc);

		this.client.removeDocument(TEST_COLLECTION_PATH, "namespaces.xml");
		assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
	}
	
	public void testDocumentWithSimpleNameSpacePrefixesSAX() throws Exception
	{
		assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));

		final String testDocument = fetchTestDocument("namespace-simpleprefixes.xml");
		this.client.insertDocument(TEST_COLLECTION_PATH, "namespaces.xml", testDocument);

		final StringWriter out = new StringWriter();
		final ContentHandler serializer = createSerializer(out);

		this.client.getDocumentAsSax(TEST_COLLECTION_PATH, "namespaces.xml", serializer);
		String doc = out.toString();

		assertNotNull(doc);
		assertXMLEqual(testDocument, doc);

		this.client.removeDocument(TEST_COLLECTION_PATH, "namespaces.xml");
		assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
	}

	public void testDocumentWithChangingNameSpacePrefixes() throws Exception
	{
		assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));

		final String testDocument = fetchTestDocument("namespace-changingprefixes.xml");
		this.client.insertDocument(TEST_COLLECTION_PATH, "namespaces.xml", testDocument);

		String doc = this.client.getDocument(TEST_COLLECTION_PATH, "namespaces.xml");
		assertNotNull(doc);
		assertXMLEqual(testDocument, doc);

		this.client.removeDocument(TEST_COLLECTION_PATH, "namespaces.xml");
		assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
	}	
   
	public void testDocumentWithChangingNameSpacePrefixesSAX() throws Exception
	{
		assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));

		final String testDocument = fetchTestDocument("namespace-changingprefixes.xml");
		this.client.insertDocument(TEST_COLLECTION_PATH, "namespaces.xml", testDocument);
		
		final StringWriter out = new StringWriter();
		final ContentHandler serializer = createSerializer(out);

		this.client.getDocumentAsSax(TEST_COLLECTION_PATH, "namespaces.xml", serializer);
		String doc = out.toString();

		assertNotNull(doc);
		assertXMLEqual(testDocument, doc);

		this.client.removeDocument(TEST_COLLECTION_PATH, "namespaces.xml");
		assertEquals(0, this.client.countDocument(TEST_COLLECTION_PATH));
	}	

	private String fetchTestDocument(String name) throws IOException, SAXException, ParserConfigurationException
	{
		final StringWriter out = new StringWriter();
		final ContentHandler serializer = createSerializer(out);
		
		parseTestDocument(name, serializer);
		
		return out.toString(); 
	}

	private void parseTestDocument(String name, ContentHandler handler) throws IOException, SAXException, ParserConfigurationException
	{
		InputStream in = getClass().getResourceAsStream("testdocs/" + name);

		if (null == in)
		{
			throw new FileNotFoundException("The resource " + name + " could not be found");
		}

		final SAXParserFactory factory = SAXParserFactory.newInstance();
		factory.setNamespaceAware(true);

		final XMLReader saxReader = factory.newSAXParser().getXMLReader();
				
		saxReader.setContentHandler(handler);		
		saxReader.parse(new InputSource(in));
	}
	
	private ContentHandler createSerializer(final Writer out)
	{
		final ContentHandler serializer = new StringSerializer()
		{
			public void endDocument() throws SAXException
			{
				super.endDocument();
				
				try
				{
					out.write(this.toString());
					out.flush();
				}
				catch (IOException e)
				{
					throw new RuntimeException("Unexpected IO exception:" + e.getMessage());
				}
			}
		};

		return serializer;
	}

	protected void assertXMLEqual(String control, String result) throws Exception
	{
		try {
			XMLAssert.assertXMLEqual(control, result);
		} catch (SAXException e) {
			fail(e.getLocalizedMessage());
		}
	}
}
