/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: MetaTest.java,v 1.3 2003/08/13 18:54:22 vgritsenko Exp $
 */

package org.apache.xindice.integration.client.services;

import org.apache.xindice.client.xmldb.services.MetaService;
import org.apache.xindice.core.meta.MetaData;
import org.apache.xindice.integration.client.AbstractXmlDbClientTest;

import org.xmldb.api.base.Collection;

/**
 * Test MetaService
 *
 * @version CVS $Revision: 1.3 $, $Date: 2003/08/13 18:54:22 $
 * @author <a href="mailto:vgritsenko@apache.org">Vadim Gritsenko</a>
 */
public class MetaTest extends AbstractXmlDbClientTest {

    static final String DOCUMENT_ID1 = "doc1";
    static final String DOCUMENT_ID2 = "doc2";

    static final String DOCUMENT1 =
            "<?xml version=\"1.0\"?>" +
            "<person>" +
            "  <first>John</first>" +
            "  <last>Smith</last>" +
            "  <age>30</age>" +
            "  <phone type=\"work\">555-345-6789</phone>" +
            "</person>";

    static final String DOCUMENT2 =
            "<?xml version=\"1.0\"?>" +
            "<person>" +
            "  <first>John</first>" +
            "  <last>Smith</last>" +
            "  <age>31</age>" +
            "  <phone type=\"work\">555-345-6789</phone>" +
            "</person>";

    static final String ATTRIBUTE_ID = "MetaDataAttribute";

    public void setUp() throws Exception {
        super.setUp();

        this.client.insertDocument(TEST_COLLECTION_PATH, DOCUMENT_ID1, DOCUMENT1);
    }

    public void tearDown() throws Exception {
        this.client.removeDocument(TEST_COLLECTION_PATH, DOCUMENT_ID1);

        super.tearDown();
    }

    public void testMetaServicePresent() throws Exception {
        // Get meta service
        Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
        MetaService service = (MetaService) col.getService("MetaService", "1.0");
        assertNotNull("MetaService is not found!", service);
    }

    public void testGetCollectionMetaData() throws Exception {
        // Get meta service
        Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
        MetaService service = (MetaService) col.getService("MetaService", "1.0");

        // Get meta data
        MetaData data = service.getMetaData();
        assertNotNull("MetaData is not found for the collection!", data);
        assertTrue("Creation time is 0!", data.getCreatedTime() != 0);
        assertTrue("Modification time is 0!", data.getLastModifiedTime() != 0);
        long created0 = data.getCreatedTime();
        long modified0 = data.getLastModifiedTime();

        // Sleep and get it again
        Thread.sleep(150);
        data = service.getMetaData();
        long created1 = data.getCreatedTime();
        long modified1 = data.getLastModifiedTime();
        assertEquals("Creation time changed on 2nd get (diff: "+ (created1 - created0) +")!", created0, created1);
        assertEquals("Modification time changed on 2nd get (diff: "+ (modified1 - modified0) +")!", modified0, modified1);

        // Sleep, add new document, and get metadata again
        Thread.sleep(150);
        this.client.insertDocument(TEST_COLLECTION_PATH, DOCUMENT_ID2, DOCUMENT2);

        data = service.getMetaData();
        assertNotNull("MetaData is not found for the collection after update!", data);
        long created2 = data.getCreatedTime();
        long modified2 = data.getLastModifiedTime();
        assertEquals("Creation time changed on update (diff: "+ (created2 - created1) +")!", created1, created2);
        assertTrue("Modification time has not changed on update!", modified1 != modified2);
    }

    public void testGetDocumentMetaData() throws Exception {
        // Get meta service
        Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
        MetaService service = (MetaService) col.getService("MetaService", "1.0");

        // Get meta data
        MetaData data = service.getMetaData(DOCUMENT_ID1);
        assertNotNull("MetaData is not found for document " + DOCUMENT_ID1 + "!", data);
        assertTrue("Creation time is 0!", data.getCreatedTime() != 0);
        assertTrue("Modification time is 0!", data.getLastModifiedTime() != 0);
        long created0 = data.getCreatedTime();
        long modified0 = data.getLastModifiedTime();

        // Sleep and get it again
        Thread.sleep(150);
        data = service.getMetaData(DOCUMENT_ID1);
        long created1 = data.getCreatedTime();
        long modified1 = data.getLastModifiedTime();
        assertEquals("Creation time changed on 2nd get (diff: "+ (created1 - created0) +")!", created0, created1);
        assertEquals("Modification time changed on 2nd get (diff: "+ (modified1 - modified0) +")!", modified0, modified1);

        // Sleep and update the document and get data again
        Thread.sleep(150);
        this.client.updateDocument(TEST_COLLECTION_PATH, DOCUMENT_ID1, DOCUMENT2);

        data = service.getMetaData(DOCUMENT_ID1);
        assertNotNull("MetaData is not found for document " + DOCUMENT_ID1 + " after update!", data);
        long created2 = data.getCreatedTime();
        long modified2 = data.getLastModifiedTime();
        assertEquals("Creation time changed on update (diff: "+ (created2 - created1) +")!", created1, created2);
        assertTrue("Modification time has not changed on update!", modified1 != modified2);
    }

    public void testSetCollectionMetaData() throws Exception {
        // Get meta service
        Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
        MetaService service = (MetaService) col.getService("MetaService", "1.0");

        // Get meta data
        MetaData data = service.getMetaData();
        long created0 = data.getCreatedTime();
        long modified0 = data.getLastModifiedTime();

        // Sleep and set metadata
        Thread.sleep(150);
        data.setContext(created0 - 500, modified0 - 500);
        service.setMetaData(data);

        // Get metadata again
        data = service.getMetaData();
        long created1 = data.getCreatedTime();
        long modified1 = data.getLastModifiedTime();
        assertEquals("Creation time changed on set (diff: "+ (created1 - created0) +")!", created0, created1);
        assertEquals("Modification time changed on set (diff: "+ (modified1 - modified0) +")!", modified0, modified1);
    }

    public void testSetDocumentMetaData() throws Exception {
        // Get meta service
        Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
        MetaService service = (MetaService) col.getService("MetaService", "1.0");

        // Get meta data
        MetaData data = service.getMetaData(DOCUMENT_ID1);
        long created0 = data.getCreatedTime();
        long modified0 = data.getLastModifiedTime();

        // Sleep and set metadata
        Thread.sleep(150);
        data.setContext(created0 - 500, modified0 - 500);
        service.setMetaData(DOCUMENT_ID1, data);

        // Get metadata again
        data = service.getMetaData(DOCUMENT_ID1);
        long created1 = data.getCreatedTime();
        long modified1 = data.getLastModifiedTime();
        assertEquals("Creation time changed on set (diff: "+ (created1 - created0) +")!", created0, created1);
        assertEquals("Modification time changed on set (diff: "+ (modified1 - modified0) +")!", modified0, modified1);
    }

    public void testSetCollectionMetaDataProperty() throws Exception {
        // Get meta service
        Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
        MetaService service = (MetaService) col.getService("MetaService", "1.0");

        // Get meta data
        String value0 = "MetaAttributeValue" + System.currentTimeMillis();
        MetaData data = service.getMetaData();
        data.setAttribute(ATTRIBUTE_ID, value0);
        service.setMetaData(data);

        // Get metadata again
        data = service.getMetaData();
        String value1 = (String)data.getAttribute(ATTRIBUTE_ID);
        assertEquals("Attribute value does not match!", value0, value1);
    }

    public void testSetDocumentMetaDataProperty() throws Exception {
        // Get meta service
        Collection col = this.client.getCollection(TEST_COLLECTION_PATH);
        MetaService service = (MetaService) col.getService("MetaService", "1.0");

        // Get meta data
        String value0 = "MetaAttributeValue" + System.currentTimeMillis();
        MetaData data = service.getMetaData(DOCUMENT_ID1);
        data.setAttribute(ATTRIBUTE_ID, value0);
        service.setMetaData(DOCUMENT_ID1, data);

        // Get metadata again
        data = service.getMetaData(DOCUMENT_ID1);
        String value1 = (String)data.getAttribute(ATTRIBUTE_ID);
        assertEquals("Attribute value does not match!", value0, value1);
    }
}
