/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xindice" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999-2001, The dbXML
 * Group, L.L.C., http://www.dbxmlgroup.com.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * CVS $Id: ConfigurationTest.java,v 1.4 2003/12/24 13:49:53 vgritsenko Exp $
 */

package org.apache.xindice.util;

import junit.framework.TestCase;

import org.apache.xindice.xml.dom.DOMParser;
import org.apache.xindice.server.Xindice;

import org.w3c.dom.Document;

/**
 * @version CVS $Revision: 1.4 $, $Date: 2003/12/24 13:49:53 $
 * @author Vladimir R. Bossicard <vladimir@apache.org>
 */
public class ConfigurationTest extends TestCase {

    Document document;

    protected void setUp() throws Exception {
        document = DOMParser.toDocument(Xindice.DEFAULT_CONFIGURATION);
    }

    /**
     * Test getName, getChild, getAttribute, etc
     */
    public void testGet() throws Exception {
        Configuration config = new Configuration(document);
        assertEquals("xindice", config.getName());

        config = config.getChild("root-collection", false);
        assertEquals("root-collection", config.getName());
        assertEquals("./db/", config.getAttribute("dbroot"));
        assertEquals("db", config.getAttribute("name"));

        assertEquals("", config.getAttribute("does-not-exist"));
        assertEquals("some-default-value", config.getAttribute("does-not-exist", "some-default-value"));
        assertEquals(false, config.getBooleanAttribute("does-not-exist"));
        assertEquals(true, config.getBooleanAttribute("does-not-exist", true));

        config = config.getChild("queryengine", false);
        assertEquals("queryengine", config.getName());

        Configuration[] children = config.getChildren();
        assertEquals(2, children.length);
        assertEquals("resolver", children[0].getName());
        assertEquals("resolver", children[1].getName());
    }

    public void testEditReadonly() throws Exception {
        Configuration config = new Configuration(document);
        try {
            config.getChild("does-not-exist", true);
            fail("ReadOnlyException expected");
        } catch (ReadOnlyException e) {
            // Expected
        }

        try {
            config.add("does-not-exist");
            fail("ReadOnlyException expected");
        } catch (ReadOnlyException e) {
            // Expected
        }

        try {
            config.setAttribute("does-not-exist", 7);
            fail("ReadOnlyException expected");
        } catch (ReadOnlyException e) {
            // Expected
        }
    }

    public void testEdit() throws Exception {
        Configuration config = new Configuration(document, false);

        Configuration child = config.getChild("does-not-exist", true);
        assertEquals("does-not-exist", child.getName());
        assertEquals("does-not-exist", config.getChild("does-not-exist").getName());

        child.setAttribute("does-not-exist", 7);
        assertEquals(7, child.getIntAttribute("does-not-exist"));
    }
}
